/* kallshow -- Test program to show all kanji's in a font file.
 *	BGI files, Font files must be present in the current directory.
 *	This is for people who want to know how font file is packed
 *	   for program development purposes.
 *	Only tested with VGA.  Make it work if it doesn't with your
 *	   video adapter.  No executable is included in the distribution.
 *      Izumi Ohzawa, 3-2-90
 *	Not copyrighted.
 */

#include <stdio.h>
#include <dos.h>	/* for int86() */
#include <stdlib.h>
#include <alloc.h>
#include <graphics.h>

#define KN2000		1700

typedef struct {
        unsigned char pix8[32];         /* 16x16 font is 32 bytes */
} Fontpack;

int ylocoff= 400;	/* Y location to display "Offset = ... " */
int Xcur=0, Ycur=0;
int MaxX, MaxY;
int MaxRow;
int MaxColumn;
int PixelPerRow = 18;
int GraphDriver = DETECT;
int GraphMode;


int KanjiColor = 7;
char font_file[80];

Fontpack *fontptr1;                /* first 64 kbytes (32*2000 chrs) */
Fontpack *fontptr2;                /* rest (32*1472 chars) (32*KN2000) */

/* === Function to read in JIS Kanji font into RAM ====================== */
int
Read_kjfont(char *fontdir)
{
FILE *fp;
int count1, count2;
char fontfilename[128];

	strcpy(fontfilename, fontdir);	
	strcat(fontfilename, font_file);
        if( (fp = fopen(fontfilename,"rb")) == NULL)
        {
		fprintf(stderr,"Can't open font file: %s\n",fontfilename);
		fprintf(stderr, "Press any key\n");
                return(-1);
        }
        fontptr1 = (Fontpack *)malloc(2000*sizeof(Fontpack));
        if(fontptr1 == NULL)
        {
                fprintf(stderr,"malloc() for fontptr1 failed.\n");
                fclose(fp);
                return(-2);
        }
        fontptr2 = (Fontpack *)malloc(KN2000*sizeof(Fontpack));
        if(fontptr2 == NULL)
        {
                fprintf(stderr,"malloc() for fontptr2 failed.\n");
                free(fontptr1);
                fclose(fp);
                return(-3);
        }

        fprintf(stderr,"Font array allocated... Reading Font... ");
        count1 = fread( (void *)fontptr1, sizeof(Fontpack), 2000, fp);
        count2 = fread( (void *)fontptr2, sizeof(Fontpack), KN2000, fp);
        if(count1 != 2000 )
        {
                fprintf(stderr, "Font read failure -- %d %d\n", count1, count2);
                free(fontptr2);
                free(fontptr1);
                fclose(fp);
                return(-4);
        }
        fclose(fp);
        fprintf(stderr,"DONE\n");
        return(count1+count2);
}

/* === Function to display 1 Kanji at current position */
void
OutKanji(int index)     /* 0 <= index <= 3700 */
{
register int LRbits, ix;
static int *font;
static int x,y,iy;

        /* Read current position */
        x = Xcur<<3;		/* Xcur*8 */
	y = Ycur * PixelPerRow+1;

	if(index < 2000) font = (int *)&fontptr1[index];
	else font = (int *)&fontptr2[index-2000];
	setcolor(KanjiColor);

        /* display kanji -- font stored in TV scan format L->R, Top->Bottom */
	for(iy=0; iy< 16; iy++)
	{
                LRbits = font[iy];
		for(ix=0; ix< 16; ix++)
		{
		    if (LRbits & 1)
		        putpixel(ix+x, iy+y, KanjiColor);
		    LRbits >>= 1;
		}
	}
        Xcur += 2;        /* Move 2 ASCII widths for next kanji */
}

void
Draw_Mesh(void)
{
int ix, iy, iyy;

	setlinestyle(SOLID_LINE, 0xffff, NORM_WIDTH);
	setcolor(BROWN);
	for(ix=0; ix<640; ix += 160)
	{
		line(ix, 0, ix, 20*PixelPerRow);	 	
	}
	for(iy=0; iy<=MaxRow+1; iy += 5)
	{
		iyy = iy*PixelPerRow;
		line(0, iyy, 639, iyy);
	}
	line(639, 0, 639, 20*PixelPerRow);
	setcolor(GREEN);

}



/* ------- main() ----------------------------------------------- */
main(int argc, char *argv[])
{
int i, startind =0;
int ch, errcode, done = 0;
int linecount =0;
unsigned char far *nrow;
unsigned int far *vidpix;
unsigned int vpsave;
int ynew, knend;
char strbuf[80];

	nrow = MK_FP(0x40, 0x84);
	vidpix = MK_FP(0x40, 0x85);
	if(argc != 2)
	{
	 	printf("Usage:  kallshow fontfile\n");
		exit(2);
	}

	strcpy(font_file, argv[1]);

        if( (knend = Read_kjfont("")) < 0)
                goto errexit;
/*
        printf("starting index= ");
        scanf("%d", &startind);
*/
	detectgraph(&GraphDriver, &GraphMode);
	if(GraphDriver < 0)
	{
	    fprintf(stderr,"No graphics hardware detected!\n");
	    return(1);
	}

	errcode = 0;
	switch(GraphDriver)
	{
	   case EGA64:
	   case EGAMONO:
		   GraphMode = EGA64HI;
	   case EGA:
		   PixelPerRow = 16;
		   ylocoff = 340;
	   case VGA:
		   PixelPerRow = 18;
		   ylocoff = 400;
		   break;
	   case HERCMONO:
		   PixelPerRow = 16;
		   ylocoff = 340;
		   break;
	   default:
		   errcode = 1;
		   break;
	}  /* end of switch(GraphDriver) */

	if(errcode)
	{
	    fprintf(stderr,"Graphics Adapter is not VGA, EGA, or Hercules.\n");
	    fprintf(stderr,"Program can probably be made to run, but the author did\n");
	    fprintf(stderr,"not have access to the technical info. on this adapter.\n");
	    fprintf(stderr,"GraphDriver= %d,  GraphMode= %d\n",
			GraphDriver, GraphMode);
	    exit(2);
	}

        initgraph(&GraphDriver, &GraphMode, "");
	errcode = graphresult();
	if(errcode <0)
	{
	    fprintf(stderr,"initgraph() error: %s\n", grapherrormsg(errcode));
	    if(errcode == -3)
		fprintf(stderr,"Copy *.BGI files to the current directory\n");
	    exit(3);
	}

        MaxRow = 19;
	MaxColumn = 79;
        setcolor(GREEN);
	vpsave = *vidpix;
	*vidpix = PixelPerRow;

	cleardevice();
	Draw_Mesh();
	sprintf(strbuf, "Offset: %d", startind);
	outtextxy(0, ylocoff, strbuf);

        while( startind < knend)
        {
                if(Xcur > MaxColumn)
		{
			if(Ycur >= MaxRow)
                        {
                          getch();
                          cleardevice();
			  Draw_Mesh();
			  sprintf(strbuf, "Offset: %d", startind);
			  outtextxy(0, ylocoff, strbuf);
			  Ycur = 0;
                        }
			else
			{
			  Ycur++; 
			}
			Xcur = 0;
		}
                OutKanji(startind);
                startind++;
        }
errexit:
	*vidpix = vpsave;
	getch();
        closegraph();
        exit(0);
}

